

program define cmt 
	version 6.0
	set mat 800

syntax varlist(default=none) [if] [in], [model(real 2)] 

set more off

if `model'==1 {
di in gr ""
di in gr "REQUESTED MODEL: TOBIT"
di in gr "======================"
di in gr ""
}

if `model'==2 {
di in gr ""
di in gr "REQUESTED MODEL: BINARY PROBIT"
di in gr "=============================="
di in gr ""
}

if `model'==3 {
di in gr ""
di in gr "REQUESTED MODEL: ORDERED PROBIT"
di in gr "==============================="
di in gr ""
}

/* put the dependent variable in depv */
/* and the rest in regs */

	tokenize `varlist'
	local depv `1'
	macro shift
	local regs `*'

if `model'==1 {
	tempvar dfdb dfds b0 sig yc D1 D2

	qui {
	ge `D1'=`depv'==0		/* indicates censoring (at zero) */
	ge `D2'=1-`D1'

	ge `yc'=. if `depv'==0
	replace `yc'=`depv' if `depv'~=0
	}

	di in gr " "
	di in gr "Estimating Tobit Model "
	di in gr "---------------------- "
	di in gr " "

	intreg `yc' `depv' `regs' `if' `in', score(`dfdb' `dfds')


qui   {
	mat `b0'=e(b)
	local k = colsof(`b0')-2

	ge `sig'=`b0'[1,`k'+2]	/* estimated standard error of residual */
	
	tempvar xb h z zz 

/* generalised residuals u1-u4 (see Chesher and Irish) */

	predict `xb', xb

	ge `z'=`xb'/`sig'
	ge `zz'=(`depv'-`xb')/`sig' 
	ge `h'=normd(`z')/(1-normprob(`z'))

	tempvar u1 u2 u3 u4

	ge `u1'= -`D1'*`h'                +`D2'*`zz'
	ge `u2'=  `D1'*`z'*`h'            +`D2'*(`zz'^2 - 1)
	ge `u3'= -`D1'*(2+`z'^2)*`h'      +`D2'*`zz'^3
	ge `u4'=  `D1'*(3*`z'+`z'^3) *`h' +`D2'*(`zz'^4 - 3)


	tempvar d0 

	sum `dfdb'
	ge `d0'=`dfdb'

	local vlist "`d0'"	
	local mlist ""	
}
}
/* ends tobit specific */


/******* PROBITS **********/

if `model'==2 | `model'==3 {

qui {
tempvar depv2

ge `depv2'=`depv'

if `model'==2 {
replace `depv2'=`depv2'+1
}


/* check dependent variable */

	qui sum `depv2'
	local miny=r(min)
	local maxy=r(max)

	if `miny' ~= 1 {
	di in red "the lowest value of dependent variable must be 1"
	exit 198
	}

	local nc=`maxy'-`miny'+1

	if `nc' == 1 {
	di in red "no variation in dependent variable"
	exit 198
	}

	for new D1-D`nc' \ num 1/`nc': gen X=`depv2'==Y

	local j=1
	while `j'<=`nc' {
	count if `depv2'==`j'
	if r(N)==0 {
	di in red "found empty category - please rescale dependent variable"
	exit 198
	}
	local j=`j'+1
	}



/* next estimate the model */
	tempvar dfdb b0 cut
}	
	oprobit `depv2' `regs' `if' `in', score(dfdb*)


qui   {
	mat `b0'=e(b)
	local ncat=e(k_cat)
	local k = colsof(`b0')-`ncat'+1		/* number of regressors */
	mat `cut'=`b0'[1,`k'+1..colsof(`b0')]	/* contains the thresholds */

	tempvar xb h0 h1 


/* define the moments */

	predict `xb', xb					/* excludes intercept */

	local j=1
	while `j'<=`ncat' {

	tempvar u1_`j' u2_`j' u3_`j' u4_`j' 


/* lowest category */

	if `j'==1 {
	ge `u1_1' = (-1)*normd(`cut'[1,1]-`xb')/normprob(`cut'[1,1]-`xb') 
	ge `u2_1' = (`cut'[1,1]-`xb')*`u1_1' 
	ge `u3_1' = (2+(`cut'[1,1]-`xb')^2)*`u1_1' 		
	ge `u4_1' = (3*(`cut'[1,1]-`xb')+(`cut'[1,1]-`xb')^3)*`u1_1' 
	}


/* intermediate categories */

if `j'>1 & `j'<`maxy' {

ge `u1_`j'' = (-1)*( normd(`cut'[1,`j']-`xb')-normd(`cut'[1,`j'-1]-`xb') )  /* 
*/   /( normprob(`cut'[1,`j']-`xb')-normprob(`cut'[1,`j'-1]-`xb') )   

ge `u2_`j'' = ( (`cut'[1,`j'-1]-`xb')*normd(`cut'[1,`j'-1]-`xb') -  /*
*/    	    (`cut'[1,`j']  -`xb')*normd(`cut'[1,`j'] -`xb') )   /*
*/   /( normprob(`cut'[1,`j']-`xb')-normprob(`cut'[1,`j'-1]-`xb') )   
		   
ge `u3_`j'' = ( normd(`cut'[1,`j'-1]-`xb')*(2+(`cut'[1,`j'-1]-`xb')^2) - /*
*/  normd(`cut'[1,`j']-`xb')*(2+(`cut'[1,`j']-`xb')^2) ) 		/*
*/  /( normprob(`cut'[1,`j']-`xb')-normprob(`cut'[1,`j'-1]-`xb') )   

ge `u4_`j'' = ( normd(`cut'[1,`j'-1]-`xb')*	/*
*/		( 3*(`cut'[1,`j'-1]-`xb')+(`cut'[1,`j'-1]-`xb')^3 ) - /*
*/ 		    normd(`cut'[1,`j']  -`xb')*	/*
*/		( 3*(`cut'[1,`j']  -`xb')+(`cut'[1,`j']  -`xb')^3 )  )   /*
*/  /( normprob(`cut'[1,`j']-`xb')-normprob(`cut'[1,`j'-1]-`xb') )   
	}


/* highest category */

if `j'==`maxy' {

ge `u1_`j'' = (-1)*( 0-normd(`cut'[1,`j'-1]-`xb') )  /* 
*/   /( 1 - normprob(`cut'[1,`j'-1]-`xb') )   

ge `u2_`j'' = ( (`cut'[1,`j'-1]-`xb')*normd(`cut'[1,`j'-1]-`xb') - 0 )   /*
*/   /( 1 - normprob(`cut'[1,`j'-1]-`xb') )   
		   
ge `u3_`j'' = ( normd(`cut'[1,`j'-1]-`xb')*(2+(`cut'[1,`j'-1]-`xb')^2) - 0 ) /*
*/  /( 1 - normprob(`cut'[1,`j'-1]-`xb') )   

ge `u4_`j'' = ( normd(`cut'[1,`j'-1]-`xb')*	/*
*/		( 3*(`cut'[1,`j'-1]-`xb')+(`cut'[1,`j'-1]-`xb')^3 ) - 0  )   /*
*/  /( 1 - normprob(`cut'[1,`j'-1]-`xb') )   
	}

	local j=`j'+1
	}


	tempvar u1 u2 u3 u4 

	ge `u1'=0
	ge `u2'=0
	ge `u3'=0
	ge `u4'=0

	local j=1
	while `j'<=`ncat' {
	replace `u1' = `u1' + D`j'*`u1_`j''
	replace `u2' = `u2' + D`j'*`u2_`j''
	replace `u3' = `u3' + D`j'*`u3_`j''
	replace `u4' = `u4' + D`j'*`u4_`j''
	local j=`j'+1
	}

}	/* quietly ends */
}	/* probit ends */



qui {
	local vlist ""	
	local mlist ""	


	local j=1
	while `j'<=`k' {
	tempvar d`j'
	tempvar mom`j'
	local j=`j'+1
	}


	if `model'==1 {

	local j=0
	tokenize `regs'

	while "`1'"~="" {
	local j=`j'+1
	ge `d`j''=`dfdb'*`1'
	ge `mom`j''=`u2'*`1'
	local vlist "`vlist' `d`j''"	
	local mlist "`mlist' `mom`j''"	
	macro shift
	}


	/* add on constant and sigma */
	local vlist "`vlist' `dfdb' `dfds'"			
	}

	if `model'>1 {

	local j=0
	tokenize `regs'
					
	while "`1'"~="" {
	local j=`j'+1
	ge `d`j''=dfdb0*`1'	/* interact dfdb with each regressor=>foc logL */

	ge `mom`j''=`u2'*`1'		/* part for heterosk. */

	local vlist "`vlist' `d`j''"	
	local mlist "`mlist' `mom`j''"	
	macro shift
	}

	local j=1
	while `j'<=(`nc'-1) {
	local vlist "`vlist' dfdb`j'"		/* add on scores for thresholds */
	local j=`j'+1
	}
	}

	noi di in gr " "

	tempvar const
	ge `const'=1

	local q=0

	while `q'<=4 {

	if `q'==0 {	

	local j=1
	tokenize `regs'

	noi di in gr " "
	noi di in gr "Separate CM tests of H0: homoskedasticity "
	noi di in gr "----------------------------------------- "
	noi di in gr " "

	mat hold_p=J(`k'+1,1,99)

	while `j'<=`k' {
	reg `const' `mom`j'' `vlist', nocons
	noi di in gr " "
	noi di in gr "Regressor: "_c
	noi di in gr "``j''"
	noi di in gr " "

	scalar NR2=e(r2)*e(N)

	local df=1

	
	noi di in gr " Chi-sq.        "_c
	noi di in ye %8.5f NR2
	noi di in gr " P>|Chi-sq.|    "_c
	noi di in ye %8.5f chiprob(`df',NR2)
	noi di in gr " df:             "_c
	noi di in ye `df'


	local j=`j'+1
	}
	}

	if `q'==1 {
	noi di in gr "Joint CM test of H0: homoskedasticity "
	noi di in gr "------------------------------------- "
	noi di in gr " "

	reg `const' `mlist' `vlist', nocons

	}

	if `q'==2 {
	noi di in gr "CM test of H0: no skewness"
	noi di in gr "------------------------- "
	noi di in gr " "

	reg `const' `u3' `vlist', nocons

	}

	if `q'==3 {
	noi di in gr "CM test of H0: no kurtosis"
	noi di in gr "------------------------- "
	noi di in gr " "

	reg `const' `u4' `vlist', nocons

	}

	if `q'==4 {
	noi di in gr "CM test of H0: no kurtosis, no skewness (normality)"
	noi di in gr "---------------------------------------------------"
	noi di in gr " "

	reg `const' `u3' `u4' `vlist', nocons

	}

	scalar NR2=e(r2)*e(N)

	if `q'==1 {
	local df=`k'
	}
	if `q'==2 {
	local df=1
	}
	if `q'==3 {
	local df=1
	}
	if `q'==4 {
	local df=2
	}

	if `q'>0 {	
	noi di in gr " Chi-sq.        "_c
	noi di in ye %8.5f NR2
	noi di in gr " P>|Chi-sq.|    "_c
	noi di in ye %8.5f chiprob(`df',NR2)
	noi di in gr " df:             "_c
	noi di in ye `df'
	}

	if `q'==1 {
	scalar phom= chiprob(`df',NR2)
	}

	if `q'==4 {
	scalar pnor= chiprob(`df',NR2)
	}

	noi di in gr " "
	noi di in gr " "

	local q=`q'+1
	}

if `model'>1 {
	drop dfdb* D*
}


	end


